A{% if A %}B{% else %}b{% endif %}{% if B %}A{% else %}a{% endif %}{% if C %}C{% else %}c{% endif %}{% if D %}A{% else %}a{% endif %}{% if E %}B{% else %}b{% endif %}{% if F %}A{% else %}a{% endif %}
