{% ifnotequal INT2 INT3 %}A{% endifnotequal %}{% ifnotequal S1 S3 %}B{% endifnotequal %}{% ifnotequal A2 A3 %}A{% endifnotequal %}{% ifnotequal H1 H3 %}C{% endifnotequal %}{% ifnotequal INT1 INT2 %}x{% else %}A{% endifnotequal %}{% ifnotequal S2 S1 %}x{% else %}C{% endifnotequal %}{% ifnotequal A2 A1 %}x{% else %}A{% endifnotequal %}{% ifnotequal H1 H2 %}x{% else %}B{% endifnotequal %}{% ifnotequal H1 A3 %}A{% else %}x{% endifnotequal %}
