{% ifequal INT1 INT2 %}A{% endifequal %}{% ifequal S1 S2 %}B{% endifequal %}{% ifequal A1 A2 %}A{% endifequal %}{% ifequal H1 H2 %}C{% endifequal %}{% ifequal INT1 INT3 %}x{% else %}A{% endifequal %}{% ifequal S1 S3 %}x{% else %}C{% endifequal %}{% ifequal A1 A3 %}x{% else %}A{% endifequal %}{% ifequal H1 H3 %}x{% else %}B{% endifequal %}{% ifequal H3 A3 %}A{% else %}x{% endifequal %}
